package model;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.sql.Date;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;

import javax.swing.JOptionPane;

import main.experimental.ui.ShowRJPGDialog;
import main.experimental.ui.ShowRTextDialog;

import org.springframework.jdbc.core.JdbcTemplate;

import tools.db.DAO;
import tools.util.Constant;
import tools.util.SubTask;
import vo.SettingData;

/**
 * Experiment 是處理 實驗，問卷以及問題 的進入點。 關於實驗本身的所有操作都在這個 class 中，與資料庫的對應也在包含在這裡。
 */
public class Experiment {
	
	static String TABLE = "questionexper";
	static String[] COL_NAMES = { "id", "title", "positive", "negative",
			"phrase", "ifnum", "questionnum", "guidetitle", "date",
			"guidesentence" };

	private int id = -1;
	private String title; // not null
	private String positive; // not null
	private String negative; // not null
	private Date date; // not null

	private int factorId = -1; // change from factorId to a factor object
								// reference
	private int fakeQuestionNum = 0;

	private String guideTitle; // not null
	private String guideGuide; // not null

	private ArrayList<Questionnaire> questionnaires;
	private String fileSepar = System.getProperty("file.separator");

	/**
	 * 復原資料庫中的所有實驗。 此 method 不應由使用者直接呼叫，應該由實驗的管理者*來呼叫， 然後透過實驗的管理者來取得需要的實驗。 * 在
	 * MiniJudge 中，實驗的管理者就是 ExperimentsTableModel。
	 */
	static ArrayList<Experiment> restoreAllExperiments() {
		ArrayList<Experiment> ret = new ArrayList<Experiment>();
		try {
			ResultSet rs = DAO.select(String.format(
					"select * from %s order by %s desc;", TABLE, COL_NAMES[0]));

			while (rs.next()) {
				Date d = rs.getDate(9);
				Experiment e = new Experiment(rs.getString(2), rs.getString(3),
						rs.getString(4), rs.getInt(5), rs.getInt(6), rs
								.getString(8), rs.getString(10), d);

				e.id = rs.getInt(1);
				e.questionnaires = Questionnaire.restoreQuestionnaires(e);

				ret.add(e);
			}
		} catch (SQLException e) {
			// e.printStackTrace();
			throw new RuntimeException(
					"Experiment.restoreAllExperiments: e.getMessage="
							+ e.getMessage());
		} finally {
			DAO.close();
		}
		return ret;
	}

	/**
	 * 正常情況下請勿使用此 constructor 傳入的參數只做簡單的賦值而已
	 */
	private Experiment(String title, String pos, String neg, int factorId,
			int fakeQuestNb, String gTitle, String gContent, Date date) {
		if (null == title || null == pos || null == neg || null == gTitle
				|| null == gContent || null == date) {
			throw new RuntimeException(
					String
							.format(
									"experiment: some parameters must not null! "
											+ "title=%s pos=%s neg=%s gTitle=%s gContent=%s date=%s",
									title, pos, neg, gTitle, gContent, date));
		}

		this.title = title;
		this.positive = pos;
		this.negative = neg;
		this.factorId = factorId;
		this.fakeQuestionNum = fakeQuestNb;
		this.guideTitle = gTitle;
		this.guideGuide = gContent;
		this.date = date;
	}

	/**
	 * 建立實驗之後，記得在 實驗的管理者 裡紀錄參照，例如：ExperimentsTableModel， 並注意其順序。
	 */
	public Experiment(String title, String positive, String negative,
			int factorId, int fakeQuestionNum, String guideTitle,
			String guideGuide, int questionnaireNum) {
		this(title, positive, negative, factorId, fakeQuestionNum, guideTitle,
				guideGuide, new Date(Calendar.getInstance().getTimeInMillis()));
		this.questionnaires = new ArrayList<Questionnaire>();

		// caution! call insert before create questionnaires!
		// because we need the id (to create questionnaires)!
		// if we can calculate the id by ourselves,
		// then, the order of this invoke doesn't matter anymore.
		insert();
	}

	/*
	 * 請勿使用此 method，此 method 是專門搭配 constructor 用的
	 */
	private int insert() {
		assert (0 > id);

		int i = DAO.insert(TABLE, COL_NAMES, new Object[] { title, positive,
				negative, factorId, fakeQuestionNum, "-1", guideTitle, date,
				guideGuide });
		if (1 != i) {
			throw new RuntimeException(
					"Error on insert a new experiment into the database!");
		}

		try {
			// get id
			// TODO this might have a problem when someone else inserted another
			// record between our insert and select
			ResultSet rs = DAO.select(String.format(
					"select %s from %s order by %s desc limit 1;",
					COL_NAMES[0], TABLE, COL_NAMES[0]));
			rs.next();
			id = rs.getInt(1);
		} catch (SQLException e) {
			// e.printStackTrace();
			throw new RuntimeException("experiment.insert: DAO.select. "
					+ e.getMessage());
		} finally {
			DAO.close();
		}
		return id;
	}

	// -------------------------------------------------------------------

	/**
	 * 如同新增實驗，記得在 實驗的管理者 那邊移除參照
	 */
	public void delete() {
		while (questionnaires.size() > 0) {
			questionnaires.get(0).delete();
		}

		int i = DAO.delete(TABLE, new String[] { COL_NAMES[0] },
				new Object[] { id });
		if (i != 1) {
			throw new RuntimeException("experiment.delete: DAO.delete. id="
					+ id + " (maybe it is deleted by someone else)");
		}
		
		JdbcTemplate jdbcTemplate = DAO.getJdbcTemplate();
		jdbcTemplate.update("delete from cache_phrase where questionexper_id = ?", id);
	}

	/**
	 * 記得在 sync 之後，通知 實驗的管理者 實驗狀態已更新
	 */
	private void sync() {
		assert (id >= 0);
		
		int i = DAO.update(TABLE, new String[] { COL_NAMES[1] },
				new Object[] { title }, id);
		if (1 != i) {
			throw new RuntimeException("experiment.sync: DAO.update. id=" + id
					+ " (maybe it is deleted by someone else)");
		}
	}

	/**
	 * 更改實驗的標題 注意：更改過後會同步更新資料庫
	 */
	public void setTitle(String title) {
		if (null == title) {
			throw new RuntimeException(
					"experiment.setTitle: title must not be null!");
		}

		this.title = title;
		sync();
	}
	
	/**
	 * 產生 r.data 的資料
	 */
	public void generateRData(String path){
		File file = new File(path);
		//檢查是否重覆，重覆的話是否要覆寫?
		if(file.exists()){
			boolean overwrite = false;
			overwrite = (JOptionPane.showConfirmDialog(null, Constant.MSG_IS_OVER_WRITE, "", JOptionPane.YES_NO_OPTION)==0) ? true : false;
			if(!overwrite){
				return;
			}
		}
		//要覆寫or沒重覆的話產生資料
		generateData(path);
		//完成的訊息
		JOptionPane.showMessageDialog(null, Constant.MSG_DATA_EXPORTED_SUCCESSFUL);
	}
	
	/**
	 * 產生給 R script 使用的 CSV 數據檔 ( r.data )
	 * 
	 */
	public void generateData(String path) {
		File file = new File(path);
		//產生 r.data 檔
		try {
			FileWriter fw = new FileWriter(file);
			fw.write(getCSVString());
			fw.flush();
			fw.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * 取得資料庫中 rFile 的路徑，因為這裡預設 r.data 產生的路徑會跟 r.r 一樣。
	 * @return 資料庫中rFile路徑/r.data
	 */
	private String getRFilePath(){
		String path = "";
		if(System.getProperty("os.name").toLowerCase().startsWith("windows")) {
			SettingModel model = new SettingModel();
			SettingData data = model.select(Constant.RFILE);
			String rFile = data.getValue();
			path = rFile.substring(0, rFile.lastIndexOf(fileSepar) + 1) + Constant.R_DATA;
		}else{
			SettingModel model = new SettingModel();
			SettingData data = model.select(Constant.RFILE);
			String rFile = data.getValue();
			String tmpStr = rFile.substring(0, rFile.lastIndexOf(fileSepar));
			path = tmpStr.substring(0, tmpStr.lastIndexOf(fileSepar) + 1) + Constant.R_DATA;
		}
		
		return path;
	}
	
	/**
	 * 執行分析實驗
	 */
	public void analyze(){
		//XXX 如果 R 相關欄位有所增減，這邊判斷條件就要增減
		SettingModel model = new SettingModel();
		if ((System.getProperty("os.name").toLowerCase().startsWith("windows") && "".equals(model.select(Constant.R).getValue()))
				|| "".equals(model.select(Constant.RFILE).getValue())
				|| "".equals(model.select(Constant.RRESULT).getValue())) {
			JOptionPane.showMessageDialog(null, Constant.MSG_NOT_SET_R,
					Constant.ERROR_MSG_ERROR, JOptionPane.ERROR_MESSAGE);
			return;
		}
		
		//在這裡產生 r.data
		generateData(getRFilePath());
		//在這裡執行 r
		try {
			String cmdStr = "";
			if(System.getProperty("os.name").toLowerCase().startsWith("windows")){
				//在這裡產生 r.bat
				buildRBatch(model);
				cmdStr = "cmd /c " + Constant.R_BAT;
			}else{
				cmdStr = "R -f " + model.select(Constant.RFILE).getValue();
				System.out.println("SH : " + cmdStr);
			}
			
			if(!"".equals(cmdStr)){
				runR(model, cmdStr);
			}
		} catch (IOException e) {
			e.printStackTrace();
		} catch (InterruptedException e) {
			e.printStackTrace();
		} finally {
			deleteFile(Constant.R_BAT);
			deleteFile(getRFilePath());
		}
	}
	
	private void runR(SettingModel model, String cmdStr) throws IOException, InterruptedException{
		StringBuffer outputSb = new StringBuffer();
		StringBuffer errorSb = new StringBuffer();
		
		String rFilePath = "";
		if (System.getProperty("os.name").toLowerCase().startsWith("windows")) {
			SettingData data = model.select(Constant.RFILE);
			String rFile = data.getValue();
			rFilePath = rFile.substring(0, rFile.lastIndexOf(fileSepar));
		}else{
			String rFile = getRFilePath();
			rFilePath = rFile.substring(0, rFile.lastIndexOf(fileSepar));
		}
		
		Process p = Runtime.getRuntime().exec(cmdStr);
		// 因為 Process 正常執行時是使用 getInputStream() 去輸出訊息，
		// 但若遇到錯誤時，便會將輸出轉到 getErrorStream() 去輸出訊息，
		// 但是如果沒有用 thread 的話會停住，原因是 getInputStream 要等待換行符號
		// 可是有問題的話，輸出轉至 ErrorStream ，因此會一直等不到換行符號而停止運作，
		// 所以用 Thread 的方式來避免此問題！。
		new Thread(new SubTask(p.getInputStream(), outputSb)).start();
		new Thread(new SubTask(p.getErrorStream(), errorSb)).start();
		p.waitFor();
		
		Calendar now = Calendar.getInstance();
		DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
		
		String resultPath = model.select(Constant.RRESULT).getValue();
		//若有產生圖檔，顯示。
		File source = new File(rFilePath + fileSepar + Constant.R_RESULT_IMAGE);
		File target = null;
		
		if(source.exists()){
			if(rFilePath.equals(resultPath)){
				new ShowRJPGDialog(source);
			}else{
				target = new File(resultPath + fileSepar + title + "_" + dateFormat.format(now.getTime()) + "_" + Constant.R_RESULT_IMAGE);
				fileCopy(source, target);
				new ShowRJPGDialog(target);
			}
		}

		//若有產生文字檔，顯示。
		source = new File(rFilePath + fileSepar + Constant.R_ANALYSIS);
		if(source.exists()){
			if(rFilePath.equals(resultPath)){
				new ShowRTextDialog(source);
			}else{
				target = new File(resultPath + fileSepar + title + "_" + dateFormat.format(now.getTime()) + "_" + Constant.R_ANALYSIS);
				fileCopy(source, target);
				new ShowRTextDialog(target);
			}
		}
		
		File dir = new File(rFilePath + fileSepar);
		String[] files = dir.list();
		for(String file : files){
			if(!file.equals("r.data") && file.endsWith(".data") && !rFilePath.equals(resultPath)){
				source = new File(rFilePath + fileSepar + file);
				target = new File(resultPath + fileSepar + file);
				fileCopy(source, target);
				break;
			}
		}
	}
	
	private void fileCopy(File source, File target) throws IOException{
		if(target.exists()){
			target.delete();
		}
		BufferedInputStream bis = new BufferedInputStream(new FileInputStream(source), 4096);
		BufferedOutputStream bos = new BufferedOutputStream(new FileOutputStream(target), 4096);
		
		int line;
		while((line=bis.read()) != -1){
			bos.write(line);	
		}
		bos.close();
		bis.close();
		source.delete();
	}
	
	/**
	 * 從資料庫取出與 R 有關的設定並將這些設定組成批次檔。
	 * @param model SettingModel
	 */
	private void buildRBatch(SettingModel model) {
		SettingData data = model.select(Constant.R);
		String r = data.getValue();
		data = model.select(Constant.RFILE);
		String rf = data.getValue();

		String nl = System.getProperty("line.separator");
		try {
			FileWriter writer = new FileWriter("R.bat");
			
			String str = "cd " + rf.substring(0, rf.lastIndexOf(fileSepar)) + nl
					+ "\"" + r + "\""
					+ " -q -f " + "\"" + rf + "\"";
			System.out.println("BAT : " + str);
			writer.write(str);
			writer.flush();
			writer.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * 刪除檔案 <br/>
	 */
	private void deleteFile(String filePath) {
		File file = new File(filePath);
		if(file.exists()){
			file.delete();
		}
	}

	private String getCSVString() {
		String nl = System.getProperty("line.separator");
		String tmp = "";
		String str = "";
		for (Questionnaire Q : questionnaires) {
			// 跳過未完成的問卷
			if (!Q.getState().equals(Questionnaire.State.COMPLETED)) {
				continue;
			}
			for (Question q : Q.getQuestions()) {
				
				// output data
				if (!q.isFake()) {
					String factor2 = "";
					if(q.getFactor2() == 0){
						factor2 = "";
					}else{
						tmp = "\tFactor2";
						factor2 = String.valueOf(q.getFactor2());
					}
					str += String.format("%s\t%s\t%s\t%s\t%s\t%s\t%s", q
							.getSpeaker(), q.getItem(), q.getSet(), q
							.getOrder(), q.getFactor1(), factor2, q
							.getJudgement());
					str += nl;
				}
			}
		}
		str = "Speaker\tItem\tSet\tOrder\tFactor1" + tmp + "\tJudgment"
			+ nl + str; // print header
		return str;
	}

	// -------------------------------------------------------------------

	/**
	 * 取得屬於此實驗的所有問卷 注意：請勿直接新增移除問卷！ 欲新增問卷請使用 Questionnaire 的 constructor，欲刪除則使用
	 * delete。 新增刪除之後不必手動更新實驗裡頭的問卷連結
	 */
	public ArrayList<Questionnaire> getQuestionnaires() {
		return questionnaires;
	}

	public int getId() {
		return id;
	}

	public String getTitle() {
		return title;
	}

	public String getPositive() {
		return positive;
	}

	public String getNegative() {
		return negative;
	}

	public int getFactorId() {
		return factorId;
	}

	public int getFakeQuestionNum() {
		return fakeQuestionNum;
	}

	public String getGuideTitle() {
		return guideTitle;
	}

	public String getGuideGuide() {
		return guideGuide;
	}

	public Date getDate() {
		return date;
	}

}