package model;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import javax.swing.table.AbstractTableModel;

import org.springframework.dao.IncorrectResultSizeDataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.PreparedStatementCreator;
import org.springframework.jdbc.support.GeneratedKeyHolder;
import org.springframework.jdbc.support.KeyHolder;

import tools.db.DAO;
import tools.db.DBConnector;
import tools.util.Constant;

/**
 * 指導句
 */
@SuppressWarnings("serial")
public class GuidesModel extends AbstractTableModel {

	static final String[] COL_NAMES = {"no", "id", "title", "guild"};
	static final String TABLE = "guildsentence";
	private static JdbcTemplate jdbcTemplate = DAO.getJdbcTemplate();
	private static GuidesModel singleton = new GuidesModel();
	
	/**
	 * 取得指導語的建構子
	 * @return 指導語本身
	 */
	public static GuidesModel getTableModel() {
		return singleton;
	}
	
	private GuidesModel() {
	}
	
	// ----------------------------------------------------------------
	/**
	 * 新增指導語
	 * @param title 指導語標題
	 * @param guide 指題語句子內容
	 * @return insert 結果。 -1 => Error or 已有存在的值 , 若為其他值則為該筆新增資料的ID
	 */
	public static int insert(final String title, final String guide) {
		
		if(-1 != getId(title, guide)){
			return -1;
		}else{
			KeyHolder keyHolder = new GeneratedKeyHolder();
			jdbcTemplate.update(new PreparedStatementCreator(){
				public PreparedStatement createPreparedStatement(Connection conn)
						throws SQLException {
					PreparedStatement ps = conn.prepareStatement("insert into " + TABLE + " values (null, ?, ?);");
					ps.setObject(1, title);
					ps.setObject(2, guide);
		            return ps;
				}}, keyHolder);
			getTableModel().fireTableRowsInserted(getTableModel().getRowCount(), getTableModel().getRowCount());
			return keyHolder.getKey().intValue();
		}
	}
	
	/**
	 * 更新指導語
	 * @param id 要更新的指導語ID
	 * @param title	指導語標題
	 * @param guide	指題語句子內容
	 * @return update結果。True, 更新成功。False, 更新失敗。
	 */
	public static boolean update(int id, String title, String guide) {
		int k = getId(title, guide);
		if(-1 != k){
			if(k == id){
				return true;
			}
			return false;
		}
		int i = DAO.update(TABLE,new String[]{COL_NAMES[2],COL_NAMES[3]}, new Object[]{title,guide}, id);
		if (1 == i) {
			int j = getTableModel().getIndex(id);
			getTableModel().fireTableRowsUpdated(j, j);
			return true;
		}
		return false;
	}

	/**
	 * 刪除指導語
	 * @param id 要刪除的指導語ID
	 * @return delete結果。True, 刪除成功。False, 刪除失敗。
	 */
	public static boolean delete(int id) {
		int index = getTableModel().getIndex(id);
		int i = DAO.delete(TABLE, id);
		if (1 == i) {
			getTableModel().fireTableRowsDeleted(index, index);
			return true;
		}
		return false;
	}

	// ----------------------------------------------------------------
	
	private static int getId(String title, String guide) {
		JdbcTemplate jt = DBConnector.getJdbcTemplate();
		try {
			return jt.queryForInt(String.format(
					"select %s from %s where %s=? and %s=?;", COL_NAMES[1],
					TABLE, COL_NAMES[2], COL_NAMES[3]), (Object) title,
					(Object) guide);
		} catch (IncorrectResultSizeDataAccessException e) {
		}
		return -1;
	}
	
	/**
	 * 取得指導語
	 * @param id 想要取得的指導語ID
	 * @return 指導語句子內容
	 */
	public static String getGuide(int id) {
		JdbcTemplate jt = DBConnector.getJdbcTemplate();
		try {
			return (String) jt.queryForObject(
					String.format("select %s from %s where %s=?;", COL_NAMES[3], TABLE, COL_NAMES[1]), 
					new Object[] {id}, String.class);
		} catch (IncorrectResultSizeDataAccessException e) {
		}
		return null;
	}

	/**
	 * 取得指導語
	 * @param id 想要取得的指導語ID
	 * @return 指導語標題
	 */
	public static String getTitle(int id) {
		try {
			ResultSet rs = DAO.select("select " + COL_NAMES[2] + " from " + TABLE + " where " + COL_NAMES[1] + "=" + id + ";");
			rs.next();
			return rs.getString(1);
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			DAO.close();
		}
		return null;
	}
	
	// ----------------------------------------------------------------
	/**
	 * 由第幾個row去推敲原本的ID
	 */
 	private int getIndex(int id) {
 			try {
 	 			ResultSet rs = DAO.select("select " + COL_NAMES[1] + " from " + TABLE + " order by "+ COL_NAMES[1] +" desc;");
 	 			int ret = -1;
				while (rs.next()) {
					++ret;
					if (rs.getInt(1) == id) {
						return ret;
					}
				}
			} catch (SQLException e) {
				e.printStackTrace();
			} finally {
				DAO.close();
			}
			return -1;
 	}
 	
 	/**
 	 * 取得該筆資料的指導語ID
 	 * @param index 該筆資料的index
 	 * @return 指導語ID
 	 */
	public int getId(int index) {
		try {
			ResultSet rs = DAO.select("select " + COL_NAMES[1] + " from " + TABLE + " order by "+ COL_NAMES[1] +" desc;");
			for (; index >= 0; --index) {
				rs.next();
			}
			return rs.getInt(1);
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			DAO.close();
		}
		return 0;
	}

	// ----------------------------------------------------------------
	
	
	
	
	@Override
	public int getColumnCount() {
		return COL_NAMES.length;
	}
	
	private static final String[] I18N_COL_NAMES = {Constant.NUMOF, Constant.ID, Constant.GUIDES_TITLE, Constant.GUIDES_GUIDE};
	@Override
	public String getColumnName(int column) {
		return I18N_COL_NAMES[column];
	}

	@Override
	public int getRowCount() {
		int count = 0;
		try {
			ResultSet rs = DAO.select("select " + COL_NAMES[1] + " from " + TABLE + ";");
			while (rs.next()) {
				++count;
			}
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			DAO.close();
		}
		return count;
	}
	
	@Override
	public Object getValueAt(int arg0, int arg1) {
		if(arg1 == 0){
			return arg0+1;
		}
		
		try {
			ResultSet rs = DAO.select("select " + COL_NAMES[arg1] + " from " + TABLE + " order by "+ COL_NAMES[1] +" desc;");
			for (; arg0 >= 0; --arg0) {
				rs.next();
			}
			return rs.getString(1);
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			DAO.close();
		}
		return null;
	}

	@Override
	public int findColumn(String columnName) {
		return super.findColumn(columnName);
	}
}
