package model;

import java.sql.Date;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.List;
import java.util.Random;

import tools.db.DAO;

/**
 * 問卷
 */
public class Questionnaire {
	
	public static enum State {NEW, COMPLETED, SENT, PRINT};
	
	static String TABLE = "questionnaire";
	static String[] COL_NAMES = {"id", "notes", "state", "builddate", "experdate", "subject", "questionexper_id"};
	
	private int id = -1;
	private State state; // not null
	private Date buildDate; // not null
	private Date answerDate;
	private String notes; 
	private String subject;
	private Experiment experiment;
	private ArrayList<Question> questions;
	
	/**
	 * 從資料庫復原屬於某個實驗的所有問卷
	 * 注意：請勿直接呼叫此 method 
	 *       (一般情況下是給 Experiment 的 restoreExperiments 呼叫的)
	 */
	static ArrayList<Questionnaire> restoreQuestionnaires(Experiment exp) {
		ArrayList<Questionnaire> ret = new ArrayList<Questionnaire>();
		try {
			ResultSet rs = DAO.select(String.format("select * from %s where %s='%s' order by %s desc;",
					TABLE, COL_NAMES[6], exp.getId(), COL_NAMES[0]));
			while (rs.next()) {
				State s = State.valueOf(rs.getString(3));
				Date d = rs.getDate(4);
				Questionnaire q = new Questionnaire(exp, s, d);
				
				try {
					q.answerDate = rs.getDate(5);
				} catch (IllegalArgumentException iae) {
					//iae.printStackTrace();
					q.answerDate = null;
				}
//				if ("null".equals(rs.getString(5)) || null == rs.getString(5)) {
//					q.answerDate = null;
//				} else {
//					q.answerDate = Date.valueOf(rs.getString(5));
//				}
				q.id = rs.getInt(1);
				q.notes = rs.getString(2);
				q.subject = rs.getString(6);
				q.questions = Question.restoreQuestions(q);
				
				ret.add(q);
			}
		} catch (SQLException e) {
			//e.printStackTrace();
			throw new RuntimeException("Questionnaire.restoreQuestionnaires: e.getMessage=" + e.getMessage());
		} finally {
			DAO.close();
		}
		return ret;
	}

	/**
	 * 注意：禁止直接建立 Questionnaire
	 */
	private Questionnaire(Experiment exp, State state, Date date) {
		if (null == exp || null == state || null == date) {
			throw new RuntimeException(String.format("Questionnaire: parameters must not be null! state=%s date=%s exp=%s", state, date, exp));
		}
		
		this.experiment = exp;
		this.state = state;
		this.buildDate = date;
	}

	/**
	 * 建立屬於某個實驗的問卷
	 * 注意：問卷新增完成後會自動存入資料庫並將參照加到實驗中
	 */
	public Questionnaire(Experiment experiment) {
		this(experiment, State.NEW, new Date(Calendar.getInstance().getTimeInMillis()));

		insert();

		questions = new ArrayList<Question>();
		
		// get real questions
		ArrayList<String> realQuestions = CachePhraseModel.getExperimentalSentences(experiment.getId());
		List<String> tmpQuestions = FakeQuestionsModel.getAllQuestions();
		ArrayList<String> fakeQuestions = new ArrayList<String>();

		// randomly select fake questions and check for redundancy
		int num = experiment.getFakeQuestionNum();
		Random rand = new Random();
		while (tmpQuestions.size() > 0 && num > 0) {
			int i = rand.nextInt(tmpQuestions.size());
			String s = tmpQuestions.get(i);
			tmpQuestions.remove(s);
			
			if (!realQuestions.contains(s) && !fakeQuestions.contains(s)) {
				fakeQuestions.add(s);
				num--;
			}
		}

		// shuffle
		ArrayList<ArrayList<Object>> tmp2 = new ArrayList<ArrayList<Object>>();
		for (String s: realQuestions) {
			ArrayList<Object> tmp = new ArrayList<Object>();
			tmp.add(s);
			tmp.add(false);
			tmp2.add(tmp);
		}
		for (String s: fakeQuestions) {
			ArrayList<Object> tmp = new ArrayList<Object>();
			tmp.add(s);
			tmp.add(true);
			tmp2.add(tmp);
		}
		Collections.shuffle(tmp2);

		// transform into Questions
		// for some stupid reasons, we can not 
		// create (new) questions, and then shuffle
		// because the order has been stored into the database after create
		for (ArrayList<Object> tmp: tmp2) {
			new Question((String)tmp.get(0), (Boolean)tmp.get(1), this);
		}	
		
		experiment.getQuestionnaires().add(0, this); // @caution the index
	}

	/**
	 * 搭配 constructor 使用的 method
	 * 請勿使用
	 */
	private int insert() {
		assert(0 > id);

//		int i = DAO.insert(String.format("insert into %s values (null, %s, '%s', '%s', %s, %s, '%s');", 
//				TABLE, null == notes ? null : "'" + notes + "'", state, buildDate, null == answerDate ? null : "'" + answerDate + "'", null == subject ? null : "'" + subject + "'", experiment.getId()));
		int i = DAO.insert(TABLE, COL_NAMES, new Object[] {notes, state, buildDate, answerDate, subject, experiment.getId()});
		if (1 != i) {
			throw new RuntimeException("questionnaire.insert: DAO.insert");
		}
			
		try {
			// get id
			ResultSet rs = DAO.select(String.format("select %s from %s order by %s desc limit 1;", COL_NAMES[0], TABLE, COL_NAMES[0]));
			rs.next();
			id = rs.getInt(1);
		} catch (SQLException e) {
			//e.printStackTrace();
			throw new RuntimeException("questionnaire.insert: e.getMessage=" + e.getMessage());
		} finally {
			DAO.close();
		}
		return id;
	}

	/**
	 * 刪除此問卷
	 * 注意：會一併刪除裡頭的問題及實驗裡頭的參照
	 */
	public void delete() {
		while (questions.size() > 0) {
			questions.get(0).delete();
		}
		
		// remove from memory
		experiment.getQuestionnaires().remove(this);
		// remove from database
//		int i = DAO.delete(String.format("delete from %s where %s='%s';", 
//				TABLE, COL_NAMES[0], id));
		int i = DAO.delete(TABLE, new String[] {COL_NAMES[0]}, new Object[] {id});
		if (i != 1) {
			throw new RuntimeException("question.delete: DAO.delete id=" + id + " (maybe it is deleted by someone else)");
		}
	}
	
	// -----------------------------------------------------------------------------
	
	/**
	 * 修改問卷的內容等資料後，會自動更新至資料庫
	 */
	private void sync() {
		assert(id >= 0);
		
//		int i = DAO.update(String.format("update %s set %s=%s, %s='%s', %s=%s, %s=%s where %s='%s';", 
//				TABLE, COL_NAMES[1], null == notes ? null : "'" + notes + "'", COL_NAMES[2], state, COL_NAMES[4], null == answerDate ? null : "'" + answerDate + "'", COL_NAMES[5], null == subject ? null : "'" + subject + "'", COL_NAMES[0], id));
		int i = DAO.update(TABLE, new String[] {COL_NAMES[1], COL_NAMES[2], COL_NAMES[4], COL_NAMES[5]},
									new Object[] {notes, state, answerDate, subject}, id);
		if (1 != i) {
			throw new RuntimeException("questionnaire.sync: DAO.update id=" + id + " (maybe it is deleted by someone else)");
		}
	}
	
	public void setSubject(String subject) {
		this.subject = subject;
		sync();
	}

	public void setNotes(String notes) {
		this.notes = notes;
		sync();
	}

	/**
	 * @param state 請使用 Questionnaire.State.{NEW|COMPILED|SENT} 作為參數傳入
	 */
	public void setState(State state) {
		this.state = state;
		sync();
	}

	public void setAnswerDate(Date answerDate) {
		this.answerDate = answerDate;
		sync();
	}

	// -----------------------------------------------------------------------------
	
	public int getId() {
		return id;
	}

	public String getNotes() {
		return notes;
	}

	public State getState() {
		return state;
	}

	public Date getBuildDate() {
		return buildDate;
	}

	public Date getAnswerDate() {
		return answerDate;
	}

	public String getSubject() {
		return subject;
	}

	public ArrayList<Question> getQuestions() {
		return questions;
	}

	public Experiment getExperiment() {
		return experiment;
	}
}
