package model;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Random;

import junit.framework.TestCase;
import tools.db.DAO;

public class TestFakeQuestions extends TestCase {

	private FakeQuestionsModel fqm;
	private String uniqueFakeQuestion = "this is a fake question that should be unique, isnt it? aoeuatouhaonuht1234435456239euintoeui2345";
	private ResultSet rs;
	private Random rand = new Random();

	@Override
	protected void setUp() throws Exception {
		super.setUp();
		// 準備測試用資料
		DAO.insert(String.format("insert into %s values (null, '%s');", FakeQuestionsModel.TABLE, uniqueFakeQuestion));
		// 建立整個程序欲測試之物件
		fqm = FakeQuestionsModel.getTableModel();
	}

	/** 測試當 FakeQuestionsModel 物件產生之後，
	 是否正確反映資料庫中的資料 
	 */
	public void testRestoreFromDB() {
			try {
				// 取得資料庫中的資料及筆數
				rs = DAO.select(String.format("select count(*) from %s;", FakeQuestionsModel.TABLE));
				int rowCount;
				rs.next();
				rowCount = rs.getInt(1);
				
				// 與 fqm 比對
				assertTrue(fqm.getRowCount() == rowCount);

				// TODO 比對資料正確性
			} catch (SQLException e) {
				e.printStackTrace();
			} finally {
				DAO.close();
			}
	}
	
	/** 測試 新增干擾句
	 1. 測試新增一筆正常(不重複)的干擾句
	 2. 測試新增一筆不正常(已經存在)的干擾句 
	 */
	public void testInsertNewQuestion() {
		String unique = uniqueFakeQuestion + rand.nextInt(99999);

		// 取得新增前的筆數
		int beforeSize = fqm.getRowCount();
		// 新增拉！
		int ret = FakeQuestionsModel.insert(unique);
		System.out.println("ret=" + ret);
		// 先確認完成新增
		assertTrue(-1 != ret);
		// 檢查筆數是否增加一筆
		assertTrue(beforeSize + 1 == fqm.getRowCount());
		// 取回新增的資料並比對
		assertTrue(unique.equals(FakeQuestionsModel.getFakeQuestion(ret)));
		// 確認剛新增的資料位於資料列表的頂端
		assertTrue(Integer.parseInt(FakeQuestionsModel.getTableModel().getValueAt(0, 0).toString()) == ret);
		// 確認有寫入資料庫
		try {
			rs = DAO.select(String.format("select count(*) from %s where %s='%s' and %s='%s';", 
					FakeQuestionsModel.TABLE, FakeQuestionsModel.COL_NAMES[0], ret, FakeQuestionsModel.COL_NAMES[1], unique));
			rs.next();
			assertTrue(1 <= rs.getInt(1));
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			DAO.close();
		}
		
		// 測試重複新增
		ret = FakeQuestionsModel.insert(unique);
		// 新增重複資料應該要失敗
		assertTrue(-1 == ret);
		// 確認資料庫中無重複之unique干擾句
		rs = DAO.select(String.format("select count(*) from %s where %s='%s';", FakeQuestionsModel.TABLE, FakeQuestionsModel.COL_NAMES[1], unique));
		try {
			rs.next();
			assertTrue(1 == rs.getInt(1));
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			DAO.close();
		}
		// 刪除剛剛新增的資料
		DAO.delete(String.format("delete from %s where %s='%s';", 
				FakeQuestionsModel.TABLE, FakeQuestionsModel.COL_NAMES[1], unique));
	}
	
	/** 測試修改干擾句
	 */
	public void testEdit() {
		String newUnique = uniqueFakeQuestion + rand.nextInt(9999);
		
		// 取得干擾句及其id 
		DAO.insert(String.format("insert into %s values (null, '%s');", 
				FakeQuestionsModel.TABLE, newUnique));
		rs = DAO.select(String.format("select %s from %s where %s='%s';", 
				FakeQuestionsModel.COL_NAMES[0], FakeQuestionsModel.TABLE, FakeQuestionsModel.COL_NAMES[1], newUnique));
		int id = -1;
		try {
			rs.next();
			id = rs.getInt(1);
		} catch (SQLException e1) {
			e1.printStackTrace();
		} finally {
			DAO.close();
		}
		assertTrue(id >= 0);
		
		// 修改為某個正常獨一無二的值
		boolean ret = FakeQuestionsModel.update(id, newUnique + "2");
		assertTrue(ret);

		// 重新由id取得此干擾句，確認已修改
		assertTrue(FakeQuestionsModel.getFakeQuestion(id).equals(newUnique + "2"));
		// 直接從資料庫中抓取，確認已寫入資料庫中
		rs = DAO.select(String.format("select %s from %s where %s='%s';", 
				FakeQuestionsModel.COL_NAMES[1], FakeQuestionsModel.TABLE, FakeQuestionsModel.COL_NAMES[0], id));
		try {
			rs.next();
			assertTrue(rs.getString(1).equals(newUnique + "2"));
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			DAO.close();
		}
		
		// 將干擾句修改為某個已經存在的值
		ret = FakeQuestionsModel.update(id, uniqueFakeQuestion);
		// 確認修改失敗
		assertTrue(false == ret);
		// 確認沒有被寫入資料庫中
		rs = DAO.select(String.format("select count(*) from %s where %s='%s';", 
				FakeQuestionsModel.TABLE, FakeQuestionsModel.COL_NAMES[1], uniqueFakeQuestion));
		try {
			rs.next();
			assertTrue(1 == rs.getInt(1));
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			DAO.close();
		}
		// 刪除剛剛新增的資料
		DAO.delete(String.format("delete from %s where %s='%s';", 
				FakeQuestionsModel.TABLE, FakeQuestionsModel.COL_NAMES[1], newUnique + "2"));
	}
	
	/** 測試刪除
	 */
	public void testDelete() {
		// 取得某干擾句及其id
		String unique = uniqueFakeQuestion + rand.nextInt(9999);
		
		// 取得干擾句及其id
		DAO.insert(String.format("insert into %s values (null, '%s');", 
				FakeQuestionsModel.TABLE, unique));
		rs = DAO.select(String.format("select %s from %s where %s='%s';", 
				FakeQuestionsModel.COL_NAMES[0], FakeQuestionsModel.TABLE, FakeQuestionsModel.COL_NAMES[1], unique));
		int id = -1;
		try {
			rs.next();
			id = rs.getInt(1);
		} catch (SQLException e1) {
			e1.printStackTrace();
		} finally {
			DAO.close();
		}
		assertTrue(id >= 0);
		
		// 刪除之，確認回傳值正常
		boolean ret = FakeQuestionsModel.delete(id);
		assertTrue(true == ret);
		
		// 透過 id and/or 句子本身 試圖取回來，確認結果為失敗的
		String s = FakeQuestionsModel.getFakeQuestion(id);
		assertTrue(null == s);
		
		// 確認資料庫中的資料已被刪除
		rs = DAO.select(String.format("select count(*) from %s where %s='%s';", 
				FakeQuestionsModel.TABLE, FakeQuestionsModel.COL_NAMES[0], id));
		try {
			rs.next();
			assertTrue(0 == rs.getInt(1));
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			DAO.close();
		}
		
		// 刪除另外一個不存在之id and/or 句子
		ret = FakeQuestionsModel.delete(id);
		assertTrue(false == ret);
	}
	
	@Override
	protected void tearDown() throws Exception {
		super.tearDown();
		DAO.delete(String.format("delete from %s where %s='%s';", 
				FakeQuestionsModel.TABLE, FakeQuestionsModel.COL_NAMES[1], uniqueFakeQuestion));
	}

}
